# coding: utf-8
"""
    InductivaWebAPI

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from inductiva.client import schemas  # noqa: F401


class OutputArchiveInfo(schemas.DictSchema):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    class MetaOapg:
        required = {
            "size",
            "contents",
        }

        class properties:
            size = schemas.IntSchema

            class contents(schemas.ListSchema):

                class MetaOapg:

                    @staticmethod
                    def items() -> typing.Type['FileInfo']:
                        return FileInfo

                def __new__(
                    cls,
                    _arg: typing.Union[typing.Tuple['FileInfo'],
                                       typing.List['FileInfo']],
                    _configuration: typing.Optional[
                        schemas.Configuration] = None,
                ) -> 'contents':
                    return super().__new__(
                        cls,
                        _arg,
                        _configuration=_configuration,
                    )

                def __getitem__(self, i: int) -> 'FileInfo':
                    return super().__getitem__(i)

            __annotations__ = {
                "size": size,
                "contents": contents,
            }

    size: MetaOapg.properties.size
    contents: MetaOapg.properties.contents

    @typing.overload
    def __getitem__(
            self, name: typing_extensions.Literal["size"]
    ) -> MetaOapg.properties.size:
        ...

    @typing.overload
    def __getitem__(
        self, name: typing_extensions.Literal["contents"]
    ) -> MetaOapg.properties.contents:
        ...

    @typing.overload
    def __getitem__(self, name: str) -> schemas.UnsetAnyTypeSchema:
        ...

    def __getitem__(self, name: typing.Union[typing_extensions.Literal[
        "size",
        "contents",
    ], str]):
        # dict_instance[name] accessor
        return super().__getitem__(name)

    @typing.overload
    def get_item_oapg(
            self, name: typing_extensions.Literal["size"]
    ) -> MetaOapg.properties.size:
        ...

    @typing.overload
    def get_item_oapg(
        self, name: typing_extensions.Literal["contents"]
    ) -> MetaOapg.properties.contents:
        ...

    @typing.overload
    def get_item_oapg(
            self, name: str
    ) -> typing.Union[schemas.UnsetAnyTypeSchema, schemas.Unset]:
        ...

    def get_item_oapg(self, name: typing.Union[typing_extensions.Literal[
        "size",
        "contents",
    ], str]):
        return super().get_item_oapg(name)

    def __new__(
        cls,
        *_args: typing.Union[
            dict,
            frozendict.frozendict,
        ],
        size: typing.Union[
            MetaOapg.properties.size,
            decimal.Decimal,
            int,
        ],
        contents: typing.Union[
            MetaOapg.properties.contents,
            list,
            tuple,
        ],
        _configuration: typing.Optional[schemas.Configuration] = None,
        **kwargs: typing.Union[schemas.AnyTypeSchema, dict,
                               frozendict.frozendict, str, date, datetime,
                               uuid.UUID, int, float, decimal.Decimal, None,
                               list, tuple, bytes],
    ) -> 'OutputArchiveInfo':
        return super().__new__(
            cls,
            *_args,
            size=size,
            contents=contents,
            _configuration=_configuration,
            **kwargs,
        )


from inductiva.client.model.file_info import FileInfo
